#!/usr/bin/perl
##############################################################################
# mtb-test.pl - Demonstrates a Gtk2::Ex::MarkupTextBuffer
# Copyright (C) 2005 Kevin C. Krinke <kckrinke@opendoorsoftware.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################
use strict;
use warnings;
# use diagnostics;

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Define constants
#%

#: truth
use constant TRUE      => 1;
use constant FALSE     => 0;
#: Exit Values
use constant EV_NORMAL => 0;
use constant EV_ERROR  => 1;
use constant EV_PANIC  => 254;
#: VerBosity levels
use constant VB_NORMAL => 1;
use constant VB_EXTRA  => 2;
use constant VB_DEBUG  => 3;

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Pre-Load modules
#%

use File::Basename;
use Getopt::Long qw( :config no_ignore_case bundling );
use Pod::Usage;

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Set main scope variables
#%

#: Variables
my $APP_TITLE = 'MarkupTextBuffer Test';
my $APP_NAME = basename( $0, '.pl' );
my $VERSION = '0.00';
my $AUTHOR = 'Kevin C. Krinke <kckrinke@opendoorsoftware.com>';
my $AUTHOR_STR = ( $APP_NAME . " (version " . $VERSION . ")\n".
                   "Author: " . $AUTHOR . "\n" .
                   "`" . $APP_NAME . " --man` for further details.\n" );

my $QUIET = 0;
my $VERBOSE = VB_NORMAL;

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Parse and validate the command line options
#%

GetOptions( 'u|?|usage'  => sub { pod2usage(); exit( EV_ERROR ); },
            'h|help'  => sub { pod2usage( 1 ); exit( EV_ERROR ); },
			'man' => sub { pod2usage( -verbose => 2 ); exit( EV_ERROR ); },
            'v|verbose+' => \$VERBOSE,
            'q|quiet!' => \$QUIET,
            'd|debug' => sub { $VERBOSE = VB_DEBUG; },
            'V|only-version' => sub { STDOUT( $VERSION ); exit( EV_NORMAL ); },
            'version' => sub { STDOUT( $AUTHOR_STR ); exit( EV_NORMAL ); }
          ) or pod2usage( 2 );

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Post-Load modules, objects and initialisations
#%

use Gtk2 -init;
#use lib '../lib';
use Gtk2::Ex::MarkupTextBuffer;

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Main application
#%

my $win = new Gtk2::Window ( 'toplevel' );
$win->signal_connect( 'delete-event' => sub { main_quit Gtk2; } );
$win->set_default_size( 600, 440 );
my $scroll = new Gtk2::ScrolledWindow;
$win->add( $scroll );
my $view = new Gtk2::TextView;
$scroll->add( $view );
my $buffer = new Gtk2::Ex::MarkupTextBuffer ();
my $test_string = qq(
<i>Italic</i>
<u>Underline</u>
<b>Bold</b>
<s>Strikethrough</s>
<big>BIG</big>
<small>small</small>
<tt>monospaced</tt>
<sup>superscript</sup><sub>subscript</sub>
<span background="grey">This should have a grey background (span)</span>
<span foreground="green">This should have a green foreground (span)</span>
<tag style="italic">This should be Italic (tag)</tag>
);
$buffer->set_markup( $test_string );
$view->set_buffer( $buffer );

#%
#% Main Loop
#%

$win->show_all();
main Gtk2;

#%
#% Main exiting normally
#%

exit( EV_NORMAL );

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Sub-Functions
#%

#%
#% Base Sub-Functions
#%

#::: undef = PANIC( SCALAR[, SCALAR, ...] )
#: print messages on STDERR, prefixed with the caller's line number
#: if the verbosity is appropriate. Uses STDERR(). Exit the application
#: with the EV_PANIC value. Newline characters are suffixed to each line.
sub PANIC {
    my @LINES = ( @_ );
    my ( $pkg, $file, $ln ) = caller();
    foreach my $line ( @LINES ) {
        STDERR( '*PANIC*[' . $ln . ']: ' . $line . "\n" );
    }
    exit( EV_PANIC );
}

#::: undef = NOTIFY( [INTEGER, ]SCALAR[, SCALAR, ...] )
#: print messages on STDOUT if the verbosity is appropriate. Uses STDOUT().
#: Newline characters are suffixed to each line.
sub NOTIFY {
    my $level = shift();
    my @LINES = ( @_ );
    unless ( _Is_Verbosity_Level( $level ) ) {
        unshift( @LINES, $level );
        $level = VB_NORMAL;
    }
    unless ( not _Is_Verbose_Enough( $level ) ) {
        foreach my $line ( @LINES ) {
            chomp( $line );
            STDOUT( $line . "\n" );
        }
    }
}

#::: undef = DEBUG( SCALAR[, SCALAR, ...] )
#: print messages on STDERR, prefixed with the caller's line number
#: if the verbosity is appropriate. Uses STDERR(). Newline characters
#: are suffixed to each line.
sub DEBUG {
    return() unless _Is_Verbose_Enough( VB_DEBUG );
    my @LINES = ( @_ );
    my ( $pkg, $file, $ln ) = caller();
    foreach my $line ( @LINES ) {
        chomp( $line );
        STDERR( 'DEBUG[' . $ln . ']: ' . $line . "\n" );
    }
}

#::: undef = ERROR( SCALAR[, SCALAR, ...] )
#: print messages on STDERR, prefixed with the caller's line number
#: if the verbosity is appropriate. Uses STDERR(). Newline characters
#: are suffixed to each line.
sub ERROR {
    my @LINES = ( @_ );
    my ( $pkg, $file, $ln ) = caller();
    foreach my $line ( @LINES ) {
        chomp( $line );
        STDERR( 'ERROR[' . $ln . ']: ' . $line . "\n" );
    }
}

#::: undef = STDERR( SCALAR[, SCALAR, ...] )
#: for each given scalar, print it on STDERR regardless of verbosity.
#: each line is not changed in any way.
sub STDERR {
    my @LINES = ( @_ );
    foreach my $line ( @LINES ) {
        print STDERR $line;
    }
}

#::: undef = STDOUT( SCALAR[, SCALAR, ...] )
#: for each given scalar, print it on STDOUT regardless of verbosity.
#: each line is not changed in any way.
sub STDOUT {
    my @LINES = ( @_ );
    foreach my $line ( @LINES ) {
        print STDOUT $line;
    }
}

#%
#% Private Sub-Functions
#%

#::: BOOL = _Is_Quiet_Verbosity( )
#: private method for verbosity management. returns true or false
#: if the current verbosity is QUIET.
sub _Is_Quiet_Verbosity {
    return( TRUE ) if $QUIET;
    return( FALSE );
}

#::: BOOL = _Is_Verbose_Enough( INTEGER )
#: private method for verbosity management, tells the truth about
#: the current verbosity situation.
sub _Is_Verbose_Enough {
    #: if we're quiet, don't do anything
    return( FALSE ) if _Is_Quiet_Verbosity();
    my $level = $_[0] || VB_NORMAL;
    return( TRUE ) if ( $level <= $VERBOSE );
    return( FALSE );
}

#::: BOOL = _Is_Verbosity_Level( INTEGER )
#: private method ofr verbosity management. Tells whether or not
#: the level given is a valid level or not.
sub _Is_Verbosity_Level {
    my $level = $_[0] || FALSE;
    return( FALSE ) unless $level =~ m!^\d$!;
    return( TRUE ) if ( $level == VB_NORMAL );
    return( TRUE ) if ( $level == VB_EXTRA );
    return( TRUE ) if ( $level == VB_DEBUG );
    return( FALSE );
}


__END__


=head1 NAME

mtb - Demonstrate a Gtk2::Ex::MarkupTextBuffer

=head1 SYNOPSIS

sample [options]

Options:

    -v --verbose         increment verbosity
    -q --quiet           run silently
    -d --debug           run with full verbosity
    -u -? --usage        brief usage message
    -h --help            brief help message
    --man                full documentation
    -V --only-version    version number
    --version            version and author message

=head1 OPTIONS

=over 4

=item B<-v --verbose>

Increment the verbosity level. There are technically four verbosity levels:

 0 - Quiet: run as silently as possible (--quiet)
 1 - Normal: print only the standard messages (default)
 2 - Extra: print standard + extra detail messages (--verbose)
 3 - Debug: print everything there is to say (-vv or --debug)

=item B<-q --quiet>

Be as quiet as possible. Usefull for automation scripting purposes.

=item B<-d --debug>

Print all levels of verbosity.

=item B<-u -? --usage>

Print the usage message and exit.

=item B<-h --help>

Print the usage with detail and exit.

=item B<--man>

Print the full manual page and exit.

=item B<-V --only-version>

Print only the version number with no newline character. Usefull for scripting
tests.

=item B<--version>

Print the application name, version, author and a message indicating to check
the full documentation for more details.

=back

=head1 file

Optional file names may be provided.

=head1 DESCRIPTION

B<mtb> is really not as complicated as this skeleton script is.

=head1 BUGS

Plenty, let the author know when you've found them.

=head1 AUTHOR

Kevin C. Krinke <kckrinke@opendoorsoftware.com>

=head1 COPYRIGHT

 mtb-test.pl - Demonstrates a Gtk2::Ex::MarkupTextBuffer
 Copyright (C) 2005 Kevin C. Krinke <kckrinke@opendoorsoftware.com>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=cut
