package Gtk2::Ex::MarkupTextBuffer;
##############################################################################
# Gtk2::Ex::MarkupTextBuffer - Apply markup text in a Gtk2::TextBuffer
# Copyright (C) 2005  Kevin C. Krinke <kckrinke@opendoorsoftware.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
##############################################################################
use strict;
use warnings;
# use diagnostics;

use constant TRUE => 1;
use constant FALSE => 0;

use Gtk2;
use Gtk2::Pango;
use XML::Parser;
use base 'Gtk2::TextBuffer';

BEGIN {
    use vars qw( $VERSION );
    $VERSION = '0.00';
}

sub new {
    my $proto = shift();
    my $class = ref( $proto ) || $proto;
    my $opts = ( ( @_ % 2 == 0 ) ? { @_ } :
                 ( ( @_ == 1 && ref( $_[0] ) eq 'Gtk2::TextTagTable' ) ?
                   { text_tag_table => $_[0] } :
                   {}
                 )
               );
    my $table = ( ( ref( $opts->{text_tag_table} ) eq 'Gtk2::TextTagTable' ) ?
                  $opts->{text_tag_table} : new Gtk2::TextTagTable
                );
#    $opts->{'wrap-mode'} ||= 'none';
	my $self = bless( new Gtk2::TextBuffer ( $table ), $class );
    $self->{opts} = $opts;
    $self->{text_tag_table} = $table;
    $self->{xml_parser} = new XML::Parser ( Style => 'Tree' );
    delete( $opts->{text_tag_table} );
	return( $self );
}

sub set_markup {
    my $self = $_[0];
    my $markup = $_[1];
    return( FALSE ) unless $markup;

    my $iter = $self->get_start_iter();
    my $xml_text = '<gtk2_ex_markuptextbuffer>' . $markup . '</gtk2_ex_markuptextbuffer>';
    my $xml_tree = $self->{xml_parser}->parse( $xml_text );

    $self->_set_markup( $iter, $xml_tree, 'gtk2_ex_markuptextbuffer', %{ $self->{opts} } );

    return( TRUE );
}

sub _set_markup {
    my $self = shift();
    my $iter = shift();
    my $xml_tree = shift();
    my $parent_tag_name = shift() || 'top';
    my %attributes = ( @_ % 2 == 0 ) ? ( @_ ) : ( );
    while ( @{ $xml_tree } ) {
        my $tag_name = shift( @{ $xml_tree } );
        my $tag_cont = shift( @{ $xml_tree } );
        last unless defined $tag_name && defined $tag_cont;
        if ( ref( $tag_cont ) eq "ARRAY" ) {
            my $tag_attr = shift( @{ $tag_cont } );
            foreach my $key ( keys( %attributes ) ) {
                $tag_attr->{ $key } = $attributes{ $key }
                 unless defined $tag_attr->{ $key };
            }
            $self->_set_markup( $iter, $tag_cont, $tag_name, %{ $tag_attr } );
        } else {
            my %attrs = $self->_merge_attributes( $parent_tag_name, %attributes );
            if ( keys( %attrs ) ) {
                $self->insert_with_tags( $iter,
                                         $tag_cont,
                                         $self->create_tag( undef, %attrs )
                                       );
            } else {
                $self->insert( $iter, $tag_cont );
            }
        }
    }
}
sub _merge_attributes {
    my $self = shift();
    my $tag_name = shift();
    my %attributes = ( @_ % 2 == 0 ) ? ( @_ ) : ( );
    if ( $tag_name =~ m!^\s*small\s*!i ) {
        #        small                  equivalent to <span size="smaller">
        $attributes{scale} = PANGO_SCALE_X_SMALL;
    } elsif ( $tag_name =~ m!^\s*big\s*!i ) {
        #        big                    equivalent to <span size="larger">
        $attributes{scale} = PANGO_SCALE_X_LARGE;
    } elsif ( $tag_name =~ m!^\s*sub\s*$!i ) {
        #        sub                    subscript
        #: who's got ideas?!
        $attributes{'rise-set'} = TRUE;
        $attributes{rise} = -100;
    } elsif ( $tag_name =~ m!^\s*sup\s*$!i ) {
        #        sup                    superscript
        #: who's got ideas?!
        $attributes{'rise-set'} = TRUE;
        $attributes{rise} = 100;
    } elsif ( $tag_name =~ m!^\s*tt\s*$!i ) {
        #        tt                     monospace font
        $attributes{family} = 'Monospace';
    } elsif ( $tag_name =~ m!^\s*i\s*!i ) {
        #        i                      italic
        $attributes{style} = 'italic';
    } elsif ( $tag_name =~ m!^\s*b\s*!i ) {
        #        b                      bold
        $attributes{weight} = PANGO_WEIGHT_BOLD;
    } elsif ( $tag_name =~ m!^\s*u\s*$!i ) {
        #        u                      underline
        $attributes{underline} = 'single';
    } elsif ( $tag_name =~ m!^\s*s\s*$!i ) {
        #        s                      strikethrough
        $attributes{strikethrough} = TRUE;
    }
    return( %attributes );
}

1;

__END__

=head1 NAME

Gtk2::Ex::MarkupTextBuffer - Apply markup text in a Gtk2::TextBuffer

=head1 SYNOPSIS

 use Gtk2::Ex::MarkupTextBuffer;
 my $buffer = new Gtk2::Ex::MarkupTextBuffer;
 $buffer->set_markup( '<tag foreground="blue">Blue text.</tag>' );

=head1 DESCRIPTION

A subclass of Gtk2::TextBuffer that provides one additional method to the
Gtk2::TextBuffer class; set_markup().

=head1 HIERARCHY

  Glib::Object
  +----Gtk2::TextBuffer
        +----Gtk2::Ex::MarkupTextBuffer

=head1 METHODS

=over 8

=item B<set_markup>

This parses a string of text for markup tags and attributes, deciphers
them and inserts the content with the intended affect applied.

=back

=head1 SUPPORTED MARKUP

=over 8

=item B<tag>                    Gtk2::TextTag

=item B<span>                   Pango span

=item B<b>                      bold

=item B<i>                      italic

=item B<u>                      underline

=item B<small>                  equivalent to <span size="smaller">

=item B<big>                    equivalent to <span size="larger">

=item B<s>                      strikethrough

=item B<sub>                    subscript

=item B<sup>                    superscript

=item B<tt>                     monospace font

=back

=head1 SEE ALSO

=over 8

 Gtk2::TextBuffer
 Gtk2::TextTag

=back

=head1 COPYRIGHT

=over 8

 Gtk2::Ex::MarkupTextBuffer - Apply markup text in a Gtk2::TextBuffer
 Copyright (C) 2005  Kevin C. Krinke <kckrinke@opendoorsoftware.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

=back

=cut
